type bop = Add | Subtract | Multiply | Divide | Mod | GreaterThan | LessThan |
  Equal | NotEqual | GtEqual | LtEqual | RaiseNote | RaiseOctave | BuildNote |
  BuildChord | Apply | AppendPhrases | CombinePhrases | ShiftPhrase |
  BeatDivide | BoolAnd | BoolOr

type uop = Negate | Invert | Play | Print

type literal = IntLit of int | StringLit of string | PitchLit of string |
  BoolLit of bool | RhythmLit of string

type td_type = Int | String | Boolean | Pitch | Note | Beat | Sequence |
  Chord | Phrase | Rhythm | Unspecified | Void

type typed_id = td_type * string

type esc_char = Doublequote | Newline | Tab | Carreturn | Backslash

type expr =
  | Unop of uop * expr
  | Binop of expr * bop * expr
  | Lit of literal
  | FnCall of string * expr list
  | Assign of typed_id * expr
  | Seq of expr list
  | Id of string

type stmt =
  | VarDecl of typed_id
  | IfThenElse of expr * stmt * stmt
  | For of expr * expr * expr * stmt
  | Foreach of typed_id * expr * stmt
  | While of expr * stmt
  | Return of expr option
  | Execute of expr
  | Block of stmt list
  | Empty

type func = {
  return_type : td_type;
  name : string;
  paramlist : typed_id list;
  body : stmt;
}

type program = func list
