{ open Parser
    
let process str = 
  (* converts strings from program text to have appropriate escape characters *)
  let rec convert s = 
    let len = String.length s in
    if (len < 2) then s else
      match (
        match (String.sub s 0 2) with
          | "\\n" -> "\n" 
          | "\\t" -> "\t"
          | "\\r" -> "\r" 
          | "\\\"" -> "\""
          | "\\\'" -> "\'"
          | "\\\\" -> "\\"
          | _ -> ""
      ) with
        | "" -> (String.sub s 0 1) ^ convert (String.sub s 1 (len - 1))
        | x -> x ^ convert (String.sub s 2 (len -2))
  in convert (String.sub str 1 (String.length str - 2))
}

rule token = parse
  | [' ' '\t' '\r' '\n'] { token lexbuf }
  | "/*" { comment lexbuf }
  | '+' { PLUS }
  | '-' { MINUS }
  | '*' { TIMES }
  | '/' { DIVIDE }
  | '%' { MOD }
  | "==" { EQ }
  | "!=" { NE }
  | '>' { GT }
  | '<' { LT }
  | "<=" { LTE }
  | ">=" { GTE }
  | '^' { CAROT }
  | "^^" { CAROT2 }
  | ':' { COLON }
  | "::" { COLON2 }
  | "**" { STAR2 }
  | "<<" { LEFT2 }
  | ">>" { RIGHT2 }
  | "@@" { AT2 }
  | "&&" { AND2 }
  | "||" { OR2 }
  | "//" { SLASH2 }
  | "\n" { NEWLINE }
  | "\r" { CARRETURN }
  | "\t" { TAB }
  | "\\" { BACKSLASH }
  | "\"" { DOUBLEQUOTE }
  | "if" { IF }
  | "else" { ELSE }
  | "for" { FOR }
  | "foreach" { FOREACH }
  | "in" { IN }
  | "while" { WHILE }
  | "function" { FUNCTION }
  | "return" { RETURN }
  | "true" { TRUE }
  | "false" { FALSE }
  | ',' { COMMA }
  | ';' { SEMICOLON }
  | '!' { EXCLAMATION }
  | '{' { OPENBRAC }
  | '}' { CLOSEBRAC }
  | '(' { OPENPAREN }
  | ')' { CLOSEPAREN }
  | '[' { OPENSEQBRAC }
  | ']' { CLOSESEQBRAC }
  | "int" { INT }
  | "string" { STRING }
  | "bool" { BOOLEAN }
  | "beat" { BEAT }
  | "pitch" { PITCH }
  | "sequence" { SEQUENCE }
  | "note" { NOTE }
  | "chord" { CHORD }
  | "phrase" { PHRASE }
  | "rhythm" { RHYTHM }
  | "void" { VOID }
  | ['0'-'9']+ as lxm { INTLIT(int_of_string lxm) }
  | '"' ([^'"']|"\\\"")* '"' as lxm { STRLIT (process lxm) }
  | '\'' ['1' '0' ' ' '-']* '\'' as lxm { RHYTHMLIT (process lxm) }
  | '$'['A'-'G' '_']['b' '#']?['0'-'9']? as lxm { PITCHLIT(lxm) }
  | ['a'-'z' 'A'-'Z' '_']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { IDENT(lxm) }
  | '=' { ASSIGN }
  | eof { EOF }
      
and comment = parse
  | "*/" { token lexbuf }
  | _ { comment lexbuf }
