import os, sys, subprocess as sub

num_errors = 0
num_failed = 0
num_warnings = 0
num_passed = 0

def run_tests():

    print '--------------------------'
    print 'Automatic Regression Tests'
    print '--------------------------'
    for root, dirs, files in os.walk('tests'):

        # ignore subversion directories
        if root.find('.svn') == -1:
            
            # test any file that ends in .tdf
            for file in files:
                if file.endswith('.tdf'):
                    filename = os.path.join(root, file)
                    test_file(filename)
                    
    # Report results
    if num_errors == 0 and num_failed == 0 and num_warnings == 0:
        print '\nALL ' + str(num_passed) +  ' TESTS PASSED!'
    else:
        print '\nThere are ' + str(num_failed) + ' failures,' , \
              str(num_errors) + ' errors and', \
              str(num_warnings) + ' warnings.'
    
    print '--------------------------'

def test_file(filename):

    global num_errors, num_warnings, num_failed, num_passed

    infile = open(filename)

    # Add switch here to directly execute file
    # Pipe the input so we can analyze it
    proc = sub.Popen(['./tonedef'], stdin=infile, stdout=sub.PIPE, stderr=sub.PIPE)
    
    # Check if there was an error running tonedef
    errors = proc.stderr.readlines()
    if len(errors) > 0:
        print "ERROR: Running Tonedef failed on '" + filename + "',  Output was:"
        print ''.join(errors)
        num_errors = num_errors + 1
        return

    gold_name = filename.replace('.tdf', '.gold')

    # make sure the gold file exists, open it, and compare.
    if not os.path.isfile(gold_name):
        print 'WARNING: The file "' + gold_name + '" does not exist. Skipping'
        num_warnings = num_warnings + 1
        return 
    
    with open(gold_name, 'r') as gold:
        expected = gold.readlines()
        actual = proc.stdout.readlines()
        
        # workaround for windows files.. strip all '\r' 
        expected = map(lambda x: x.replace('\r', ''), expected)
        actual = map(lambda x: x.replace('\r', ''), actual)

        if expected != actual:
            print 'FAIL: ' + filename
            num_failed = num_failed + 1

            # For debugging
            if len(sys.argv) > 1 and sys.argv[1] == '-v':
                print 'Expected:'
                print expected
                print ''
                print 'Actual:'
                print actual
                print ''

            return 
    
    # If control gets here, we passed
    print 'PASS: ' + filename
    num_passed = num_passed + 1
    return 0

if __name__ == '__main__':
    
    # Make sure that everything is compiled first
    if not os.path.isfile('tonedef'):
        sub.Popen('make').wait()
    run_tests()
